# ifndef _CAMERA_POINTS
# define _CAMERA_POINTS

struct CameraPoints
{
public:
  float x,y,z;
  CameraPoints() {}

  CameraPoints(float X, float Y, float Z) {
    x =X; y = Y; z = Z;
  }

  void Set_Points(float X, float Y, float Z) {
    x =X; y = Y; z = Z;
  }

  CameraPoints operator=(CameraPoints CP){
    x = CP.x; y = CP.y; z = CP.z;
    return CameraPoints(x,y,z);
  }

  CameraPoints operator-(CameraPoints CP) {
     return CameraPoints(x - CP.x, y - CP.y, z - CP.z);
  }

  CameraPoints operator+(CameraPoints CP) {
     return CameraPoints(x + CP.x, y + CP.y, z + CP.z);
  }

  CameraPoints operator*(float num) {
     return CameraPoints(x * num, y * num, z * num);
  }

  CameraPoints operator/(float num) {
     return CameraPoints(x / num, y / num, z / num);
  }

  CameraPoints Cross(CameraPoints CP) {
    CameraPoints cross_result;

    // The X value for the vector is:  (V1.y * V2.z) - (V1.z * V2.y)
    cross_result.x = ((y * CP.z) - (z * CP.y));

    // The Y value for the vector is:  (V1.z * V2.x) - (V1.x * V2.z)
    cross_result.y = ((z * CP.x) - (x * CP.z));

    // The Z value for the vector is:  (V1.x * V2.y) - (V1.y * V2.x)
    cross_result.z = ((x * CP.y) - (y * CP.x));
    return cross_result;
  }

  float Magnitude(CameraPoints vNormal)
  {
    // Here is the equation:  magnitude = sqrt(V.x^2 + V.y^2 + V.z^2)   Where V is the vector
    return (float)sqrt( (vNormal.x * vNormal.x) +
                        (vNormal.y * vNormal.y) +
                        (vNormal.z * vNormal.z) );
  }

  CameraPoints Normalize(CameraPoints vVector)
  {
    // For a vector to be normalized, it means that it has a length of 1.
    // For instance, a vector (2, 0, 0) would be (1, 0, 0) once normalized.

    // Get the magnitude of our normal
    float magnitude = Magnitude(vVector);
    // That will make our vector a total length of 1.
    vVector = vVector / magnitude;

    return vVector;
  }

};

# endif
