/*
  Frustum code   : Learned from www.Gametutorials.com       (once got to a stage to re-program I shall )
  email : ian@designersoft.co.uk
*/

#include "frustum.h"

void Frustum::NormalizePlane(int side)
{
  // magnitude = sqrt ( x^2, y ^ 2, z^2)    Normalize = 'plane' / magnitude
  float magnitude = (float)sqrt( frustum[side][A] * frustum[side][A] + frustum[side][B] * frustum[side][B] + frustum[side][C] * frustum[side][C]);
  frustum[side][A] /= magnitude;
  frustum[side][B] /= magnitude;
  frustum[side][C] /= magnitude;
  frustum[side][D] /= magnitude;
}

void Frustum::CalculateFrustum()
{
    float proj[16];     // project matrix
    float mod[16];    // modelview matrix
    float clip[16];     // clipping matrix

    // get and sort the project and model view matrix
    glGetFloatv( GL_PROJECTION_MATRIX, proj );
    glGetFloatv( GL_MODELVIEW_MATRIX, mod );

    // now multiply the matrix to get the clipping planes.

    clip[ 0] = mod[ 0] * proj[ 0] + mod[ 1] * proj[ 4] + mod[ 2] * proj[ 8] + mod[ 3] * proj[12];
    clip[ 1] = mod[ 0] * proj[ 1] + mod[ 1] * proj[ 5] + mod[ 2] * proj[ 9] + mod[ 3] * proj[13];
    clip[ 2] = mod[ 0] * proj[ 2] + mod[ 1] * proj[ 6] + mod[ 2] * proj[10] + mod[ 3] * proj[14];
    clip[ 3] = mod[ 0] * proj[ 3] + mod[ 1] * proj[ 7] + mod[ 2] * proj[11] + mod[ 3] * proj[15];

    clip[ 4] = mod[ 4] * proj[ 0] + mod[ 5] * proj[ 4] + mod[ 6] * proj[ 8] + mod[ 7] * proj[12];
    clip[ 5] = mod[ 4] * proj[ 1] + mod[ 5] * proj[ 5] + mod[ 6] * proj[ 9] + mod[ 7] * proj[13];
    clip[ 6] = mod[ 4] * proj[ 2] + mod[ 5] * proj[ 6] + mod[ 6] * proj[10] + mod[ 7] * proj[14];
    clip[ 7] = mod[ 4] * proj[ 3] + mod[ 5] * proj[ 7] + mod[ 6] * proj[11] + mod[ 7] * proj[15];

    clip[ 8] = mod[ 8] * proj[ 0] + mod[ 9] * proj[ 4] + mod[10] * proj[ 8] + mod[11] * proj[12];
    clip[ 9] = mod[ 8] * proj[ 1] + mod[ 9] * proj[ 5] + mod[10] * proj[ 9] + mod[11] * proj[13];
    clip[10] = mod[ 8] * proj[ 2] + mod[ 9] * proj[ 6] + mod[10] * proj[10] + mod[11] * proj[14];
    clip[11] = mod[ 8] * proj[ 3] + mod[ 9] * proj[ 7] + mod[10] * proj[11] + mod[11] * proj[15];

    clip[12] = mod[12] * proj[ 0] + mod[13] * proj[ 4] + mod[14] * proj[ 8] + mod[15] * proj[12];
    clip[13] = mod[12] * proj[ 1] + mod[13] * proj[ 5] + mod[14] * proj[ 9] + mod[15] * proj[13];
    clip[14] = mod[12] * proj[ 2] + mod[13] * proj[ 6] + mod[14] * proj[10] + mod[15] * proj[14];
    clip[15] = mod[12] * proj[ 3] + mod[13] * proj[ 7] + mod[14] * proj[11] + mod[15] * proj[15];

    // Now we actually want to get the sides of the frustum.  To do this we take
    // the clipping planes we received above and extract the sides from them.

    // This will extract the RIGHT side of the frustum
    frustum[RIGHT][A] = clip[ 3] - clip[ 0];
    frustum[RIGHT][B] = clip[ 7] - clip[ 4];
    frustum[RIGHT][C] = clip[11] - clip[ 8];
    frustum[RIGHT][D] = clip[15] - clip[12];

    // Now that we have a normal (A,B,C) and a distance (D) to the plane,
    // we want to normalize that normal and distance.

    // Normalize the RIGHT side
    NormalizePlane(RIGHT);

    // This will extract the LEFT side of the frustum
    frustum[LEFT][A] = clip[ 3] + clip[ 0];
    frustum[LEFT][B] = clip[ 7] + clip[ 4];
    frustum[LEFT][C] = clip[11] + clip[ 8];
    frustum[LEFT][D] = clip[15] + clip[12];

    // Normalize the LEFT side
    NormalizePlane(LEFT);

    // This will extract the BOTTOM side of the frustum
    frustum[BOTTOM][A] = clip[ 3] + clip[ 1];
    frustum[BOTTOM][B] = clip[ 7] + clip[ 5];
    frustum[BOTTOM][C] = clip[11] + clip[ 9];
    frustum[BOTTOM][D] = clip[15] + clip[13];

    // Normalize the BOTTOM side
    NormalizePlane(BOTTOM);

    // This will extract the TOP side of the frustum
    frustum[TOP][A] = clip[ 3] - clip[ 1];
    frustum[TOP][B] = clip[ 7] - clip[ 5];
    frustum[TOP][C] = clip[11] - clip[ 9];
    frustum[TOP][D] = clip[15] - clip[13];

    // Normalize the TOP side
    NormalizePlane(TOP);

    // This will extract the BACK side of the frustum
    frustum[BACK][A] = clip[ 3] - clip[ 2];
    frustum[BACK][B] = clip[ 7] - clip[ 6];
    frustum[BACK][C] = clip[11] - clip[10];
    frustum[BACK][D] = clip[15] - clip[14];

    // Normalize the BACK side
    NormalizePlane(BACK);

    // This will extract the FRONT side of the frustum
    frustum[FRONT][A] = clip[ 3] + clip[ 2];
    frustum[FRONT][B] = clip[ 7] + clip[ 6];
    frustum[FRONT][C] = clip[11] + clip[10];
    frustum[FRONT][D] = clip[15] + clip[14];

    // Normalize the FRONT side
    NormalizePlane(FRONT);
}

bool Frustum::PointInFrustum(float x, float y, float z)
{
    // Plane equation (x * x + y * y + z * z + D = 0) if on the same plane.
    // Else if >= then infront of the plane, or < then behind

    for (int i =0; i < 6; i++)
      if (frustum[i][A] * x + frustum[i][B] * y + frustum[i][C] * z + frustum[i][D] <= 0)  // behind the plane or on it
        return false;
    return true;    // if the point was inside all of the frustum (In front of all sides of the frustum (top cropped of pymaind)
}

float Frustum::PointInFrustumCal(float x,float y,float z, float size)
{
   // plane equation (x * x + y * y + z * z + D = 0) if on the same plane.
   // elseif >= then infront of plane, or < then behind
   float ret = 0;
   for (int i = 0; i < 6; i++)
     if ((ret = (frustum[i][A] * x + frustum[i][B] * y + frustum[i][C] * z + frustum[i][D])) <=0)   // ret +
       return ret;  // behind
   return ret;      // in front, but by how much.
}

bool Frustum::SphereInFrustum(float x, float y, float z, float radius)
{
    // Plane equation (x * x + y * y + z * z + D = 0) if on the same plane.
    // Else if >= then infront of the plane, or < then behind
    // Sphere radius is used to check for the whole object is inside the frustum since if the point of the sphere is -3 and radius = -5 then
    // the sphere would be in the view since -3 - -5 = 2

    for (int i =0; i < 6; i++)
      if (frustum[i][A] * x + frustum[i][B] * y + frustum[i][C] * z + frustum[i][D] <= -radius)  // behind the plane or on it
        return false;
    return true;    // if the point was inside all of the frustum (In front of all sides of the frustum (top cropped of pymaind)
}

bool Frustum::CubeInFrustum(float xyz[3], float size)
{
    return CubeInFrustum(xyz[0], xyz[1], xyz[2], size);
}

bool Frustum::CubeInFrustum(float x, float y, float z, float size)
{
    // Plane equation (x * x + y * y + z * z + D = 0) if on the same plane.
    // Else if >= then infront of the plane, or < then behind

    for (int i =0; i < 6; i++)  {
      if (frustum[i][A] * (x - size) + frustum[i][B] * (y - size) + frustum[i][C] * (z - size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x - size) + frustum[i][B] * (y + size) + frustum[i][C] * (z - size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x + size) + frustum[i][B] * (y - size) + frustum[i][C] * (z - size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x + size) + frustum[i][B] * (y + size) + frustum[i][C] * (z - size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop

      if (frustum[i][A] * (x - size) + frustum[i][B] * (y - size) + frustum[i][C] * (z + size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x - size) + frustum[i][B] * (y + size) + frustum[i][C] * (z + size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x + size) + frustum[i][B] * (y - size) + frustum[i][C] * (z + size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop
      if (frustum[i][A] * (x + size) + frustum[i][B] * (y + size) + frustum[i][C] * (z + size) + frustum[i][D] > 0) continue; // in front of the plane then goto next test in the loop

      return false;   // if get here then behind one of the planes
    }

    return true;    // if the point was inside all of the frustum (In front of all sides of the frustum (top cropped of pymaind)
}

float Frustum::CubeInFrustumCal(float x, float y, float z, float size)
{
    // Plane equation (x * x + y * y + z * z + D = 0) if on the same plane.
    // Else if >= then infront of the plane, or < then behind
    float ret;
    for (int i =0; i < 6; i++)  {
      if ((ret = (frustum[i][A] * (x - size) + frustum[i][B] * (y - size) + frustum[i][C] * (z - size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x - size) + frustum[i][B] * (y + size) + frustum[i][C] * (z - size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x + size) + frustum[i][B] * (y - size) + frustum[i][C] * (z - size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x + size) + frustum[i][B] * (y + size) + frustum[i][C] * (z - size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop

      if ((ret = (frustum[i][A] * (x - size) + frustum[i][B] * (y - size) + frustum[i][C] * (z + size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x - size) + frustum[i][B] * (y + size) + frustum[i][C] * (z + size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x + size) + frustum[i][B] * (y - size) + frustum[i][C] * (z + size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop
      if ((ret = (frustum[i][A] * (x + size) + frustum[i][B] * (y + size) + frustum[i][C] * (z + size) + frustum[i][D])) > 0) continue; // in front of the plane then goto next test in the loop

      return ret; // false;   // if get here then behind one of the planes
    }

    return ret;// true;    // if the point was inside all of the frustum (In front of all sides of the frustum (top cropped of pymaind)
}
