/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Thu May 22 10:54:42 BST 2003
    copyright            : (C) 2003 by Ian Porter
    email                : ian@designersoft.co.uk
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "templates/def_include.h"
#include "OcPoints.h"

#define OCPOINTS_SIZE 167 // 500
#define OCPOINTS_POINTS 3

void RenderScene();
void Quit(int return_value);
void CreateWindow(const char* strWindowName, int iWidth, int iHeight, int iVideoFlags);
void SetupPixelFormat(void);
void OpenGLScreen(int iWidth, int iHeight);
void HandleKeyEvent(SDL_keysym *keysym);
void Render();
void init_Grab_Inputs();

int VideoFlags = 0;                                 // Video flags for creation of window
SDL_Surface *SDL_Window = NULL;   // drawing surface for the SDL Window
Octree_2 OTree;

bool Mouse_Left = false;
bool Mouse_Right = false;
  
int main(void)
{
  bool done = false;
  SDL_Event event;
  FILE 	*Ptr;
  int i, pointsi;
  OctreePoints_2 *OcPoints = NULL;
  OcPoints = new OctreePoints_2[OCPOINTS_SIZE];
  float insert[3] = { 0.0f, 0.0f, 0.0f};

  Ptr = fopen("Terrain.raw","r");
  i =0;
  pointsi =0;
  while ( fscanf(Ptr, "%f %f %f\n", &OcPoints[i].x[pointsi], &OcPoints[i].y[pointsi], &OcPoints[i].z[pointsi]) != EOF)
   {
      printf("%f\t%f\t%f\t%i\t%i\n", OcPoints[i].x[pointsi], OcPoints[i].y[pointsi] , OcPoints[i].z[pointsi] ,i, pointsi);
      pointsi++;
      if (pointsi >= OCPOINTS_POINTS) {
          pointsi = 0;
          i++;
      }
   }

  OTree.create_node(OcPoints, OCPOINTS_SIZE, 50.0f, insert);
  fclose(Ptr);

  printf("Basic SDL_Test, Octree test\n");

  if (SDL_Init(SDL_INIT_VIDEO) < 0)       // try to initialize the SDL video module
  {
      printf("Failed SDL_INIT : %s\n", SDL_GetError() );
      return EXIT_FAILURE;
  }
  SetupPixelFormat();
  CreateWindow("Basic SDL_Text -  Octree test", SCREEN_WIDTH, SCREEN_HEIGHT, VideoFlags);
  OpenGLScreen(SCREEN_WIDTH, SCREEN_HEIGHT);

  while (!done) {
    while ( SDL_PollEvent(&event) ){
        switch (event.type) {
            case SDL_KEYDOWN :
              HandleKeyEvent(&event.key.keysym);
              break;
            case SDL_QUIT:
              done = true;
              break;

              case SDL_MOUSEBUTTONDOWN: {
                  switch (event.button.button) {
                      case SDL_BUTTON_LEFT:
                         Mouse_Left = true;
                         break;
                     case SDL_BUTTON_RIGHT:
                       Mouse_Right = true;
                        break;
                    default: break;
                }
                break;
            }

            case SDL_MOUSEBUTTONUP: {
                switch (event.button.button) {
                    case SDL_BUTTON_LEFT:
                       Mouse_Left = false;
                       break;
                    case SDL_BUTTON_RIGHT:
                       Mouse_Right = false;
                       break;
                    default: break;
                }
                break;
            }
            
            default : break;
        }
    }

    Render();
  }

  delete OcPoints;
  
  return EXIT_SUCCESS;
}

void Render()
{
    // camera viewing area (at the start),  eye         center                        up
    static Camera camera(CameraPoints(0,0,-20), CameraPoints(-10,0,0), CameraPoints(0,1,0)) ;
    static Frustum frustum;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);   // clear screen and buffer
    glLoadIdentity();    // reset matrix stack

    if (Mouse_Left) camera.Move_Camera(-mouse_speed);
    if (Mouse_Right) camera.Move_Camera(mouse_speed);

    camera.Set_View_By_Mouse();
    camera.Poistion_Camera();

    frustum.CalculateFrustum();

    glPushMatrix();
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glColor3ub( 255, 50, 50);             // red , green, blue
    glBegin (GL_TRIANGLES); //GL_POLYGON);       // begin draw
        OTree.display(frustum);
    glEnd();
    glColor3ub( 255, 255, 255);
    glBegin (GL_LINES); //GL_POLYGON);       // begin draw
        OTree.display(frustum);
    glEnd();
    glPopMatrix();

    SDL_GL_SwapBuffers();       // swap the backbuffers to foreground
}

// Create a SDL_Window  (Name, width, height, video flags)
void CreateWindow(const char* strWindowName, int iWidth, int iHeight, int iVideoFlags)
{
    SDL_Window = SDL_SetVideoMode(iWidth, iHeight, SCREEN_DEPTH, iVideoFlags);
    if (SDL_Window == NULL)
    {
        printf("Failed SDL_Window : %s\n",SDL_GetError() );
        Quit(0);
    }
    SDL_WM_SetCaption(strWindowName, strWindowName);
    //                                   window caption , icon caption
}

// Setup pixelformat for the SDL screen
void SetupPixelFormat(void)
{
  VideoFlags = SDL_OPENGL;       // OpenGl window
  VideoFlags |=SDL_HWPALETTE;   // hardware colour palette
  VideoFlags |=SDL_RESIZABLE;   // window is resizeable
//  VideoFlags |=SDL_FULLSCREEN;
  const SDL_VideoInfo *VideoInfo = SDL_GetVideoInfo();       // get the SDL information for the video hardware

  if (VideoInfo == NULL)
  {
    printf("No video information : %s\n",SDL_GetError() );
    Quit(0);
  }

  // determine hardware surface or software
  if (VideoInfo->hw_available) VideoFlags |= SDL_HWSURFACE; else VideoFlags |= SDL_SWSURFACE;
  // determine blitting (for fast copying / moving etc. (http://www.csc.liv.ac.uk/~fish/HTML/blitzman/bm_blitter.html)
  if (VideoInfo->blit_hw) VideoFlags |= SDL_HWACCEL;

  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );         // tell SDL that the GL drawing is going to be double buffered
  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   SCREEN_DEPTH);         // size of depth buffer
  SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);          // we aren't going to use the stencil buffer
  SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);        // this and the next three lines set the bits allocated per pixel -
  SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);      // - for the accumulation buffer to 0
  SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
  SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);
}

void OpenGLScreen(int iWidth, int iHeight)
{
  if (iHeight == 0) iHeight = 1;    // make sure height is not 0
  glViewport(0,0,iWidth, iHeight);    // output area (x,y,width, height)
  glMatrixMode(GL_PROJECTION);    // spefiy the matrix stack for mipulation
  glLoadIdentity();                                 // reset matrix mode with 1 0 0 0, 0 1 0 0, 0 0 1 0, 0 0 0 1
  gluPerspective(45.0f, (GLfloat)iWidth / (GLfloat)iHeight, 1, 150.0f);
  //  Field of view angle (y), aspect angle ration x(width) to y (height), nearest clipping distance, farthest distance clipping distance > 1
  glMatrixMode(GL_MODELVIEW);      // set matrix stack to model for mipulation
  glLoadIdentity();     // resest model view
}

void HandleKeyEvent(SDL_keysym *keysym)
{
  switch(keysym->sym) {
      case SDLK_ESCAPE:
        Quit(0);
      default:
        break;
  }
}

void Quit(int return_value)
{
  SDL_Quit();
  exit(return_value);
}

void init_Grab_Inputs(void) {
  SDL_WM_GrabInput(SDL_GRAB_ON);

   if ( SDL_ShowCursor(SDL_DISABLE) != SDL_DISABLE )
          printf("%s\n", SDL_GetError() );
}
/* Video Flags

      The flags parameter is the same as the flags field of  the  SDL_Surface
       structure. OR'd combinations of the following values are valid.

       SDL_SWSURFACE       Create the video surface in system memory

       SDL_HWSURFACE       Create the video surface in video memory

       SDL_ASYNCBLIT       Enables the use of asynchronous updates of the dis-
                           play surface. This will usually slow down  blitting
                           on  single  CPU  machines,  but may provide a speed
                           increase on SMP systems.

       SDL_ANYFORMAT       Normally, if a video surface of the requested bits-
                           per-pixel  (bpp) is not available, SDL will emulate
                           one with a shadow  surface.  Passing  SDL_ANYFORMAT
                           prevents  this and causes SDL to use the video sur-
                           face, regardless of its pixel depth.

       SDL_HWPALETTE       Give SDL exclusive  palette  access.  Without  this
                           flag  you  may  not  always  get the the colors you
                           request with SDL_SetColors or SDL_SetPalette.

       SDL_DOUBLEBUF       Enable hardware double buffering; only  valid  with
                           SDL_HWSURFACE.   Calling  SDL_Flip  will  flip  the
                           buffers and update the  screen.  All  drawing  will
                           take  place on the surface that is not displayed at
                           the  moment.  If  double  buffering  could  not  be
                           enabled   then   SDL_Flip   will   just  perform  a
                           SDL_UpdateRect on the entire screen.

       SDL_FULLSCREEN      SDL will attempt to use a  fullscreen  mode.  If  a
                           hardware  resolution  change  is  not possible (for
                           whatever reason), the next higher  resolution  will
                           be  used and the display window centered on a black
                           background.

       SDL_OPENGL          Create an OpenGL rendering context. You should have
                           previously   set   OpenGL   video  attributes  with
                           SDL_GL_SetAttribute.

       SDL_OPENGLBLIT      Create an OpenGL rendering context, like above, but
                           allow  normal  blitting operations. The screen (2D)
                           surface   may   have   an   alpha   channel,    and
                           SDL_UpdateRects  must  be used for updating changes

      SDL_RESIZABLE       Create a  resizable  window.  When  the  window  is
                           resized by the user a SDL_VIDEORESIZE event is gen-
                           erated and SDL_SetVideoMode  can  be  called  again
                           with the new size.

       SDL_NOFRAME         If  possible,  SDL_NOFRAME  causes  SDL to create a
                           window with  no  title  bar  or  frame  decoration.
                           Fullscreen  modes automatically have this flag set.

*/
/*
      void glMatrixMode( GLenum mode )

PARAMETERS
       mode  Specifies  which matrix stack is the target for subsequent matrix
             operations.    Three   values   are    accepted:    GL_MODELVIEW,
             GL_PROJECTION,    and   GL_TEXTURE.    The   initial   value   is
             GL_MODELVIEW.

       GL_MODELVIEW             Applies subsequent matrix  operations  to  the
                                modelview matrix stack.

       GL_PROJECTION            Applies  subsequent  matrix  operations to the
                                projection matrix stack.

       GL_TEXTURE               Applies subsequent matrix  operations  to  the
                                texture matrix stack.

       GL_COLOR                 Applies  subsequent  matrix  operations to the
                                color matrix stack.
*/
/*
       void gluLookAt( GLdouble eyeX,
                       GLdouble eyeY,
                       GLdouble eyeZ,
                       GLdouble centerX,
                       GLdouble centerY,
                       GLdouble centerZ,
                       GLdouble upX,
                       GLdouble upY,
                       GLdouble upZ )

PARAMETERS
       eyeX, eyeY, eyeZ
                       Specifies the position of the eye point.

       centerX, centerY, centerZ
                       Specifies the position of the reference point.

       upX, upY, upZ   Specifies the direction of the up vector.
*/

/*    C SPECIFICATION
       void gluSphere( GLUquadric* quad,
                       GLdouble radius,
                       GLint slices,
                       GLint stacks )

PARAMETERS
       quad    Specifies the quadrics object (created with gluNewQuadric).

       radius  Specifies the radius of the sphere.

       slices  Specifies the number of subdivisions around the z axis (similar
               to lines of longitude).

       stacks  Specifies the number of subdivisions along the z axis  (similar
               to lines of latitude).
*/
